local ml=require "ml"
-- soar: generate a standalone archive of pure lua scripts from a manifest
--
-- Reads from solua.out, writes to standard out. Depends on the input modules
-- being well-formed Lua code, which should be the case with soluadep output.

-- Arguably, this should create another loader at the end instead of
-- preload; perhaps the user has more up-to-date versions of modules
-- available.

local readfile = ml.throw(ml.readfile)
local solua_out = "return "..readfile("solua.out")
local sources = loadstring(solua_out)()
assert(sources._main, "No main chunk listed")

local out = io.stdout

local tinsert = table.insert

local binary_modules = {}
local excluded_modules = {}
for mod,path in pairs(sources) do
    if path == true then
	tinsert(excluded_modules, mod)
	sources[mod] = nil
    elseif path == 1010 then
	tinsert(binary_modules, mod)
	sources[mod] = nil
    end
end
table.sort(binary_modules)
table.sort(excluded_modules)

local _main = readfile(sources._main)
local _, tailindex, shebang = string.find(_main, "^(#!.-)\r?\n")
if shebang then
    _main = string.sub(_main, tailindex)
else
    shebang = "#!/usr/bin/env lua"
end

out:write(shebang, "\n")

local function banner(mods, b)
    if #mods > 0 then 
	out:write("--# ", b, ": ")
	for _,m in ipairs(mods) do
	    out:write(m, " ")
	end
	out:write("\n")
    end
end
banner(excluded_modules, "Modules not included")
banner(binary_modules, "Required binary modules")

-- that funny looking line is both a magic cookie and a way of confusing 
-- lua-mode.el.

out:write[==[
local function _()STANDALONE_LUA[=[]]end]=]--archive, contains modules at end
]==]

out:write(_main)
out:write([[----------------- main script ends here; referenced libraries follow
--------------------------------------------------------------------
end
function STANDALONE_LUA() end
]])

sources._main = nil

for mod,filename in pairs(sources) do
    out:write("\n----------------- begin ", mod)
    if not hide_filenames then
	out:write(" (from ", filename, ")")
    end
    out:write("\nlocal function __()\n", readfile(filename))
    out:write(ml.expand([[

end
package.preload["${mod}"] = __
----------------- end ${mod}
]], {mod=mod}))
end			    

out:write[[

-- Set the environment variable UNSOAR to instead produce the original
-- script on stdout. 
if os.getenv("UNSOAR") then
    io.input(arg[0])
    shebang = io.read()
    io.write(shebang, "\n")
    assert((string.match(io.read(), "STANDALONE_LUA%[=%[%]%]")))
    local pattern = string.gsub(
	"^----------------- main script ends here; referenced libraries follow",
	"%-", "%%-")
    while true do
	local line = io.read()
	if string.match(line, pattern) then
	    os.exit(0)
	end
	io.write(line, "\n")
    end
end

-- Invoking the main body; the script from the top of the program runs here.
_()
]]
